import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:taboola_sdk/classic/webview_manager.dart';
import 'package:taboola_sdk/classic/webview_gesture_recognizer.dart';
import 'package:taboola_sdk/utils/logger.dart';
import 'dart:io' show Platform;

class TaboolaWebView extends StatelessWidget {
  String _initialUrl;
  Set<Factory<OneSequenceGestureRecognizer>> _savedFactoryRecognizers =
      Set<Factory<OneSequenceGestureRecognizer>>();
  WebViewGestureRecognizer _vertical;
  BuildContext _buildContext;

  final WebViewManager _webViewManager;
  WebView _webView;

  final String _publisherId,
      _placement,
      _mode,
      _pageType,
      _pageUrl,
      _pageId,
      _targetType;

  TaboolaWebView(
      this._publisherId,
      this._placement,
      this._mode,
      this._pageType,
      this._pageUrl,
      this._pageId,
      this._targetType,
      this._webViewManager,
      int viewId) {
    _initialUrl = "https://cdn.taboola.com/shared/templateJS.html?"
        "publisher=${_encodeParam(_publisherId)}"
        "&mode=${_encodeParam(_mode)}"
        "&pageType=${_encodeParam(_pageType)}"
        "&placement=${_encodeParam(_placement)}"
        "&targetType=${_encodeParam(_targetType)}"
        "&pageId=${_encodeParam(_pageId)}"
        "&pageUrl=${_encodeParam(_pageUrl)}"
        "&DIV_ID=taboola2"
        "&viewID=${_encodeParam(viewId.toString())}";

    _webViewManager.taboolaWebViewDidScrollToTop =
        _taboolaWebViewDidScrollToTop;

    if (Platform.isAndroid) {
      //Only Android needs the "Scroll Squitch" solution. iOS Implements a stretching WebView without inner scrolling.
      _vertical = WebViewGestureRecognizer(this); //..onUpdate = (_) {};

      _savedFactoryRecognizers = [
        Factory<WebViewGestureRecognizer>(
          () => _vertical,
        )
      ].toSet();
    }
  }

  /// An update event called from outside this WebView. This is where screen measurements decide if to enable our scroll switch logic.
  ///
  /// _vertical - This is the vertical GestureRecognizer we implement our ScrollSwitch logic in. It's a Taboola 'WebViewGestureRecognizer' object.
  void scrollListener(
      ScrollController controller, bool isFeed, bool canEnableScroll) {
    if (!controller.hasClients) {
      Logger.log(
          "Scroll Controller is not attached to a scrolling parent yet.");
      //This might happen if controller logic is called while the relevant parent widget is still building. Source: https://stackoverflow.com/questions/52114535/scrollcontroller-not-attached-to-any-scroll-views
      return;
    }

    if (Platform.isAndroid && isFeed && canEnableScroll) {
      if (controller.offset >= controller.position.maxScrollExtent &&
          !controller.position.outOfRange) {
        Logger.log("reach the bottom");
      }
    }
    _sendNotifyExternalRects();
  }

  void _sendNotifyExternalRects() {
    if (_buildContext != null) {
      final RenderBox renderBox = _buildContext.findRenderObject();
      if (renderBox != null) {
        try {
          //Originally meant to catch an assertion failure (but no functional issue): "Flutter: Failed assertion: 'attached': is not true."
          final position = renderBox.localToGlobal(Offset.zero);
          Size windowSize = MediaQuery.of(_buildContext).size;

          Size webViewSize = Size(windowSize.width, _buildContext.size.height);
          _webViewManager.webViewRect = position & webViewSize;
          _webViewManager.nativeWindowRect = Offset(0, 0) & windowSize;
          _webViewManager.notifyExternalRects();
        } catch (error) {
          Logger.log('_sendNotifyExternalRects() | Caught error: $error');
        }
      }
    }
  }

  ///This is called from MobileLoader (through WebViewManger)
  void _taboolaWebViewDidScrollToTop() {
    if (Platform.isAndroid && _vertical != null) {
      _vertical.onWebViewScrolledToContentTop();
    }
  }

  ///Returns the offset from screen top (and left?) when the WebView is close to screen top the offset.dy will return close to 0 result.
  Offset getPositionOnScreen() {
    if (Platform.isAndroid) {
      if (_buildContext != null) {
        final RenderBox renderBoxRed = _buildContext.findRenderObject();
        if (renderBoxRed != null) {
          return renderBoxRed.localToGlobal(Offset.zero);
        }
      }
    }
    return Offset(0, 0);
  }

  @override
  Widget build(BuildContext context) {
    Logger.log(_initialUrl);
    _buildContext = context;
    _webView = WebView(
      initialUrl: _initialUrl,
      javascriptMode: JavascriptMode.unrestricted,
      onWebViewCreated: (WebViewController controller) {
        _webViewManager.controller = controller;
        _sendNotifyExternalRects();
      },
      javascriptChannels: Set.from([_webViewManager.javascriptChannel()]),
      gestureRecognizers: _savedFactoryRecognizers,
      allowsInlineMediaPlayback:
          true, //Added to avoid video items from opening fullscreen automatically (only applies for iOS, in Android it's inline=true as a default.
      debuggingEnabled: true,
    );

    return _webView;
  }

  String _encodeParam(String param) {
    if (param != null) {
      return Uri.encodeFull(param);
    }
    return "";
  }
}
